##############################################################################
#
# Copyright (c) 2002, 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test OnlineHelp

"""
import re
import os
import unittest
from doctest import DocTestSuite

from zope.interface import Interface, implementer
from zope.configuration import xmlconfig
import zope.traversing
from zope.component import testing

from zope.testing import renormalizing

from zope.app.onlinehelp import onlinehelptopic
from zope.app.onlinehelp import onlinehelp

class I1(Interface):
    pass

@implementer(I1)
class Dummy1(object):
    pass

class Dummy2(object):
    pass

class TestOnlineHelpResource(unittest.TestCase):

    def test_size(self):
        r = onlinehelptopic.OnlineHelpResource(
            os.path.join(testdir(), 'help.html'))
        self.assertEqual(20, r.getSize())

class TestBaseOnlineHelpTopic(unittest.TestCase):

    def test_bad_path(self):
        from zope.configuration.exceptions import ConfigurationError

        self.assertRaises(
            ConfigurationError,
            onlinehelptopic.BaseOnlineHelpTopic,
            'id', 'title', 'path that does not exist',
            'parentpath')

class TestOnlineHelp(unittest.TestCase):

    def test_bad_path(self):
        from zope.configuration.exceptions import ConfigurationError
        the_help = onlinehelp.OnlineHelp('title', testdir())

        self.assertRaises(
            ConfigurationError,
            the_help.registerHelpTopic,
            'parentpath', 'id', 'title',
            'path that does not exist')


class TestOnlineHelpNamespace(unittest.TestCase):

    def test_context(self):
        from zope.app.onlinehelp import globalhelp
        from zope.app.onlinehelp import helpNamespace

        traversed = helpNamespace(self).traverse(None, None)
        self.assertIs(traversed.context, self)
        self.assertIsNone(getattr(globalhelp, 'context', None))

    def test_cannot_pickle(self):
        from zope.app.onlinehelp import helpNamespace
        import pickle

        traversed = helpNamespace(self).traverse(None, None)
        self.assertRaises(TypeError,
                          pickle.dumps, traversed)


def testdir():
    return os.path.dirname(__file__)

def setUp(tests):
    testing.setUp()
    xmlconfig.file('configure.zcml', zope.traversing)

def test_suite():
    checker = renormalizing.RENormalizing((
        (re.compile(r"u('.*')"), r'\1'),
    ))

    return unittest.TestSuite((
        DocTestSuite('zope.app.onlinehelp',
                     setUp=setUp, tearDown=testing.tearDown,
                     checker=checker),
        DocTestSuite('zope.app.onlinehelp.onlinehelptopic',
                     setUp=setUp, tearDown=testing.tearDown,
                     checker=checker),
        DocTestSuite('zope.app.onlinehelp.onlinehelp',
                     setUp=setUp, tearDown=testing.tearDown,
                     checker=checker),
        unittest.defaultTestLoader.loadTestsFromName(__name__),
    ))

if __name__ == '__main__':
      unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2002, 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""`OnlineHelp` tree view

"""
__docformat__ = 'restructuredtext'

from zope.component import getUtility
from zope.i18n import translate
from zope.publisher.browser import BrowserView
from zope.traversing.api import getPath, joinPath

from zope.app.onlinehelp.interfaces import IOnlineHelp

class OnlineHelpTopicTreeView(BrowserView):
    """Online help topic tree view."""

    def __init__(self, context, request):
        super(OnlineHelpTopicTreeView, self).__init__(context, request)
        self.onlinehelp = getUtility(IOnlineHelp, "OnlineHelp")

    def getTopicTree(self):
        """Return the tree of help topics.

        We build a flat list of tpoics info dict.
        Iterate this dict oan build from the level info
        a navigation tree in the page tmeplate.
        Each time you get a level 0 means this is a subitem of the
        Onlinehelp itself::

          >>> info = [('id',{infoDict}),(),()]

          <ul class="tree" id="tree">
            <li><a href="#">items</a>
              <ul>
                <li><a href="#">item</a></li>
              </ul>
            </li>
            <li><a href="#">items</a>
              <ul>
                <li><a href="#">items</a>
                  <ul>
                    <li><a href="#">item</a></li>
                    <li><a href="#">item</a></li>
                    <li><a href="#">item</a></li>
                  </ul>
                </li>
                <li><a href="#">items</a>
                  <ul>
                    <li><a href="#">item</a></li>
                    <li id="activeTreeNode"><a href="#">active item</a></li>
                    <li><a href="#">item</a></li>
                  </ul>
                </li>
              </ul>
            </li>
          <ul>
        """
        return self.renderTree(self.onlinehelp)

    def renderTree(self, root):
        """Reder a unordered list 'ul' tree with a class name 'tree'."""
        res = []
        intend = "  "
        res.append('<ul class="tree" id="tree">')
        for topic in root.getSubTopics():
            item = self.renderLink(topic)

            # expand if context is in tree
            if self.isExpanded(topic):
                res.append('  <li class="expand">%s' % item)
            else:
                res.append('  <li>%s' % item)

            if len(topic.getSubTopics()) > 0:
                res.append(self.renderItemList(topic, intend))
            res.append('  </li>')

        res.append('<ul>')

        return '\n'.join(res)

    def renderItemList(self, topic, intend):
        """Render a 'ul' elements as childs of the 'ul' tree."""
        res = []
        intend = intend + "  "
        res.append('%s<ul>' % intend)

        for item in topic.getSubTopics():

            # expand if context is in tree
            if self.isExpanded(topic):
                res.append('  %s<li class="expand">' % intend)
            else:
                res.append('  %s<li>' % intend)

            res.append(self.renderLink(item))
            if len(item.getSubTopics()) > 0:
                res.append('    %s%s' % (
                    self.renderItemList(item, intend), intend))
            res.append('  %s</li>' % intend)
        res.append('%s</ul>' % intend)

        return '\n'.join(res)

    def renderLink(self, topic):
        """Render a href element."""
        title = translate(topic.title, context=self.request,
                default=topic.title)
        if topic.parentPath:
            url = joinPath(topic.parentPath, topic.id)
        else:
            url = topic.id
        return '<a href="/++help++/%s">%s</a>\n' % (url, title)

    def isExpanded(self, topic):
        if topic.parentPath:
            path = joinPath(topic.parentPath, topic.id)
        else:
            path = topic.id
        try:
            if getPath(self.context).startswith('/' + path):
                return True
        except:
            # TODO: fix it, functional test doesn't like getPath? ri
            pass
        return False



##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Implementation of OnlineHelp System.

This is the default implementation of the `OnlineHelp`. It defines the global
`OnlineHelp` in which all basic Zope-core help screens are registered.

"""
__docformat__ = 'restructuredtext'

import os

from zope.component import getGlobalSiteManager, getUtilitiesFor
from zope.interface import implementer
from zope.configuration.exceptions import ConfigurationError
from zope.traversing.interfaces import IContainmentRoot
from zope.traversing.api import traverse

from zope.app.onlinehelp.interfaces import IOnlineHelp, IOnlineHelpTopic
from zope.app.onlinehelp.onlinehelptopic import OnlineHelpTopic

@implementer(IOnlineHelp, IContainmentRoot)
class OnlineHelp(OnlineHelpTopic):
    """
    Implementation of :class:`~.IOnlineHelp`.

    >>> import os
    >>> from zope import component
    >>> from zope.component.interfaces import IFactory
    >>> from zope.component.factory import Factory
    >>> from zope.app.onlinehelp.tests.test_onlinehelp import testdir
    >>> from zope.app.onlinehelp.tests.test_onlinehelp import I1, Dummy1
    >>> path = os.path.join(testdir(), 'help.txt')

    Create an `OnlineHelp` instance

    >>> onlinehelp = OnlineHelp('Help', path)

    First do the interface verifying tests.

    >>> from zope.interface.verify import verifyObject
    >>> from zope.traversing.interfaces import IContainmentRoot
    >>> verifyObject(IOnlineHelp, onlinehelp)
    True
    >>> verifyObject(IContainmentRoot, onlinehelp)
    True

    Register a new subtopic for interface 'I1' and view 'view.html'

    >>> from zope.app.onlinehelp.onlinehelptopic import OnlineHelpTopic
    >>> from zope.app.onlinehelp.onlinehelptopic import RESTOnlineHelpTopic
    >>> from zope.app.onlinehelp.onlinehelptopic import STXOnlineHelpTopic
    >>> from zope.app.onlinehelp.onlinehelptopic import ZPTOnlineHelpTopic
    >>> default = Factory(OnlineHelpTopic)
    >>> rest = Factory(RESTOnlineHelpTopic)
    >>> stx = Factory(STXOnlineHelpTopic)
    >>> zpt = Factory(ZPTOnlineHelpTopic)
    >>> component.provideUtility(default, IFactory, 'onlinehelp.topic.default')
    >>> component.provideUtility(rest, IFactory, 'onlinehelp.topic.rest')
    >>> component.provideUtility(stx, IFactory, 'onlinehelp.topic.stx')
    >>> component.provideUtility(zpt, IFactory, 'onlinehelp.topic.zpt')
    >>> path = os.path.join(testdir(), 'help2.txt')
    >>> onlinehelp.registerHelpTopic('', 'help2', 'Help 2',
    ...     path, I1, 'view.html')

    Test if the subtopic is set correctly
    >>> onlinehelp['help2'].title
    'Help 2'

    Additionally it should appear as a utility
    >>> topic = component.getUtility(IOnlineHelpTopic,'help2')
    >>> topic.title
    'Help 2'

    add another topic without parent
    >>> onlinehelp.registerHelpTopic('missing', 'help3', 'Help 3',
    ...     path, I1, 'view.html')

    The new topic should not be a child of the onlinehelp instance
    >>> 'help3' in onlinehelp.keys()
    False

    But it is available as a utility
    >>> topic = component.getUtility(IOnlineHelpTopic,'missing/help3')
    >>> topic.title
    'Help 3'

    now register the missing parent
    >>> onlinehelp.registerHelpTopic('', 'missing', 'Missing',
    ...     path, I1, 'view.html')

    This is a child on the onlinehelp
    >>> 'missing' in onlinehelp.keys()
    True

    >>> missing = onlinehelp['missing']

    This topic should now have 'help3' as a child
    >>> 'help3' in missing.keys()
    True

    """

    def __init__(self, title, path):
        super(OnlineHelp, self).__init__('', title, path, None)

    def registerHelpTopic(self, parent_path, id, title,
                          doc_path, interface=None, view=None,
                          class_=None, resources=None):
        "See zope.app.onlineHelp.interfaces.IOnlineHelp"

        if not os.path.exists(doc_path):
            if doc_path.endswith('.txt') and os.path.exists(doc_path[:-4] + '.rst'):
                doc_path = doc_path[:-4] + '.rst'

        if not os.path.exists(doc_path):
            raise ConfigurationError(
                "Help Topic definition %s does not exist" % doc_path
            )

        if class_ is None:
            class_ = OnlineHelpTopic

        # Create topic base on the custom class or OnlinHelpTopic
        topic = class_(id, title, doc_path, parent_path, interface, view)

        # add resources to topic
        if resources is not None:
            topic.addResources(resources)

        # add topic to onlinehelp hierarchy
        parent = None
        try:
            parent = traverse(self, parent_path)
            parent[id] = topic
        except KeyError:
            pass

        for t in getUtilitiesFor(IOnlineHelpTopic):
            if parent is None:
                if t[1].getTopicPath() == parent_path:
                    t[1][id] = topic
            if topic.getTopicPath() == t[1].parentPath:
                topic[t[1].id] = t[1]

        # Add topic to utilities registry
        #utils = getService(Utilities)
        #utils.provideUtility(IOnlineHelpTopic, topic, topic.getTopicPath())

        getGlobalSiteManager().registerUtility(
            topic, IOnlineHelpTopic, topic.getTopicPath())



##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""OnlineHelp System.

Create the global ``OnlineHelp`` instance.

"""
__docformat__ = 'restructuredtext'

import os

from zope.component import getUtilitiesFor
from zope.interface import providedBy
from zope.proxy import ProxyBase
from zope.proxy import non_overridable
from zope.testing import cleanup

from zope.app.onlinehelp.interfaces import IOnlineHelpTopic
from zope.app.onlinehelp.onlinehelp import OnlineHelp


# Global Online Help Instance
path = os.path.join(os.path.dirname(__file__),
                      'help', 'welcome.stx')
globalhelp = OnlineHelp('Online Help', path)

class _TraversedOnlineHelpProxy(ProxyBase):
    """
    A proxy around the globalhelp object that is returned when we
    traverse through the helpNamespace.

    It adds the ``context`` attribute to the context that was traversed
    through.
    """
    __slots__ = ('context',)

    def __init__(self, context):
        self.context = context
        ProxyBase.__init__(self, globalhelp)

    @non_overridable
    def __reduce__(self, proto=None):
        raise TypeError("Not picklable")
    __reduce_ex__ = __reduce__


class helpNamespace(object):
    """ help namespace handler """

    def __init__(self, context, request=None):
        self.context = context

    def traverse(self, name, ignored):
        """
        Used to traverse to an online help topic.

        Returns a proxy for the global :class:`~.OnlineHelp` instance
        with the traversal context.
        """
        return _TraversedOnlineHelpProxy(self.context)


def getTopicFor(obj, view=None):
    """Determine topic for an object and optionally a view.

    Iterate through all directly provided Interfaces and
    see if for the interface (and view) exists a Help Topic.

    Returns the first match.

    Prepare the tests:

    >>> import os
    >>> from zope.app.onlinehelp.tests.test_onlinehelp import testdir
    >>> from zope.app.onlinehelp.tests.test_onlinehelp import I1, Dummy1, Dummy2
    >>> from zope.app.onlinehelp import tests as ztapi
    >>> from zope.component.interfaces import IFactory
    >>> from zope.component.factory import Factory
    >>> from zope.app.onlinehelp.onlinehelptopic import OnlineHelpTopic
    >>> from zope.app.onlinehelp.onlinehelptopic import RESTOnlineHelpTopic
    >>> from zope.app.onlinehelp.onlinehelptopic import STXOnlineHelpTopic
    >>> from zope.app.onlinehelp.onlinehelptopic import ZPTOnlineHelpTopic
    >>> default = Factory(OnlineHelpTopic)
    >>> rest = Factory(RESTOnlineHelpTopic)
    >>> stx = Factory(STXOnlineHelpTopic)
    >>> zpt = Factory(ZPTOnlineHelpTopic)
    >>> ztapi.provideUtility(IFactory, default, 'onlinehelp.topic.default')
    >>> ztapi.provideUtility(IFactory, rest, 'onlinehelp.topic.rest')
    >>> ztapi.provideUtility(IFactory, stx, 'onlinehelp.topic.stx')
    >>> ztapi.provideUtility(IFactory, zpt, 'onlinehelp.topic.zpt')
    >>> path = os.path.join(testdir(), 'help.txt')

    Register a help topic for the interface 'I1' and the view 'view.html'

    >>> onlinehelp = OnlineHelp('Help', path)
    >>> path = os.path.join(testdir(), 'help2.txt')
    >>> onlinehelp.registerHelpTopic('', 'help2', 'Help 2',
    ...     path, I1, 'view.html')

    The query should return it ('Dummy1' implements 'I1):

    >>> getTopicFor(Dummy1(),'view.html').title
    'Help 2'

    A query without view should not return it

    >>> getTopicFor(Dummy1()) is None
    True

    Do the registration again, but without a view:

    >>> onlinehelp = OnlineHelp('Help', path)
    >>> onlinehelp.registerHelpTopic('', 'help2', 'Help 2',
    ...     path, I1, None)
    >>> getTopicFor(Dummy1()).title
    'Help 2'

    Query with view should not match

    >>> getTopicFor(Dummy1(), 'view.html') is None
    True

    Query with an object, that does not provide 'I1' should
    also return None

    >>> getTopicFor(Dummy2()) is None
    True

    If there is a second interface also provided with the same
    view name and registered for that interface, still only the first
    topic will be found.

    >>> from zope.interface import Interface, implementer, alsoProvides
    >>> class I3(Interface):
    ...     pass
    >>> @implementer(I3)
    ... class Dummy3(object):
    ...     pass

    >>> path = os.path.join(testdir(), 'help2.txt')
    >>> onlinehelp.registerHelpTopic('a', 'help3', 'Help 3',
    ...     path, I3, None)

    >>> getTopicFor(Dummy3()).title
    'Help 3'
    >>> getTopicFor(Dummy1()).title
    'Help 2'

    >>> @implementer(I1, I3)
    ... class Dummy4(object):
    ...     pass
    >>> getTopicFor(Dummy4()).title
    'Help 2'

    >>> @implementer(I3, I1)
    ... class Dummy5(object):
    ...     pass
    >>> getTopicFor(Dummy5()).title
    'Help 3'

    """
    for interface in providedBy(obj):
        for _name, topic in getUtilitiesFor(IOnlineHelpTopic):
            if topic.interface == interface and topic.view == view:
                return topic

def _clear():
    globalhelp.__init__(globalhelp.title, globalhelp.path)


cleanup.addCleanUp(_clear)
